from datetime import datetime
import sys

class LogWriter(object):
    """LogWriter provides logging capabilities for an ongoing Black Fox optimization.

    Parameters
    ----------
    file : str
        Optional file or sys.stdout used for logging

    """

    def __init__(self, file=sys.stdout):
        self.log_file = file

    def write_neural_network_statues(self, id, statuses):
        status = statuses[-1]
        if len(statuses) >= 2 or (status.validation_set_error > 0 and status.training_set_error > 0):
            msg = ("%s - %s, "
                    "Generation: %s/%s, "
                    "Validation set %s: %f%s, "
                    "Training set %s: %f%s, "
                    "Epoch: %d, "
                    "Optimization Id: %s") % (
                datetime.now(),
                status.state,
                status.generation,
                status.total_generations,
                status.metric_name,
                status.validation_set_error,
                ' %' if status.metric_name == 'MAPE' else '',
                status.metric_name,
                status.training_set_error,
                ' %' if status.metric_name == 'MAPE' else '',
                status.epoch,
                id
            )
            self.write_string(msg)
        else: 
            self.write_string("Evaluating initial models")

    def write_random_forest_statues(self, id, statuses):
        status = statuses[-1]
        if len(statuses) >= 2 or (status.validation_set_error > 0 and status.training_set_error > 0):
            msg = ("%s - %s, "
                "Generation: %s/%s, "
                "Validation set %s: %f, "
                "Training set %s: %f, "
                "Optimization Id: %s") % (
                datetime.now(),
                status.state,
                status.generation,
                status.total_generations,
                status.metric_name,
                status.validation_set_error,
                status.metric_name,
                status.training_set_error,
                id
            )
            self.write_string(msg)
        else: 
            self.write_string("Evaluating initial models")

    def write_xgboost_statues(self, id, statuses):
        self.write_random_forest_statues(id, statuses)

    def write_string(self, msg):
        if isinstance(self.log_file, str):
            with open(self.log_file, mode='a', encoding='utf-8', buffering=1) as f:
                f.write(msg+'\n')
        else:
            self.log_file.write(msg+'\n')
            self.log_file.flush()



# from blackfox import InputConfig, Range
from blackfox_restapi.models.input_config import InputConfig
from blackfox_restapi.models.range import Range

def __fill_inputs(inputs, data_set):

    if inputs is None or len(inputs) == 0:
        inputs = []
        for row in data_set:
            for i, d in enumerate(row):
                if len(inputs) <= i or inputs[i] is None:
                    inputs.append(InputConfig(range = Range(d, d), encoding = None))
                else:
                    r = inputs[i].range
                    r.min = min(r.min, d)
                    r.max = max(r.max, d)
    else:
        if len(inputs) < len(data_set[0]) or len(inputs) > len(data_set[0]):
            raise Exception ("The number of encoding types must match the number of input variables")
        for j in range(len(inputs)):
            inp = inputs[j]
            if inp.range is None:
                inp.range = Range()
                inp.range.min = data_set[0][j]
                inp.range.max = data_set[0][j]
                for row in data_set:
                    d = row[j]
                    inp.range.min = min(inp.range.min, d)
                    inp.range.max = max(inp.range.max, d)
    
    for input in inputs:
        if input.encoding is None:
            if isinstance(input.range.max, str) or isinstance(input.range.min, str):
                input.encoding = 'Target'
                input.range.max = None
                input.range.min = None
            else:
                input.encoding = 'None'
        elif input.encoding is not 'None' and (isinstance(input.range.max, str) or isinstance(input.range.min, str)):
            input.range.max = None
            input.range.min = None
    return inputs

data = [['prvi', 1], ['prvi', 1], ['drugi', 2], ['treci', 3]]
inputs = [InputConfig(encoding='Target'), InputConfig(encoding='Target')]

__fill_inputs(inputs, data)





from datetime import datetime
import sys


class CsvLogWriter(object):

    def __init__(self, file=sys.stdout, only_change=True, clear_file=True):
        self.log_file = file
        self.write_string(
            'Time,Status,Generation,Total generations,Validation set error,Training set error,Generation time [s],StartTime,End time,Optimization Id, Epoch', clear_file)
        self.only_change = only_change
        self.previous_generation = -1

    def write_neural_network_statues(self, id, statuses):
        if len(statuses) >= 2:
            status = statuses[-2]
            if self.only_change == False or self.previous_generation != status.generation:
                self.__write_nn_status(id, status)
                if statuses[-1].state != 'Active':
                    self.__write_nn_status(id, statuses[-1])
                self.previous_generation = status.generation

    def __write_nn_status(self, id, status):
        msg = ("%s,%s,%d,%d,%f,%f,%d,%s,%s,%s,%d") % (
            datetime.now(),
            status.state,
            status.generation,
            status.total_generations,
            status.validation_set_error,
            status.training_set_error,
            status.generation_seconds,
            status.start_date_time,
            status.estimated_date_time,
            id,
            status.epoch
        )
        self.write_string(msg)

    def write_random_forest_statues(self, id, statuses):
        if len(statuses) >= 2:
            status = statuses[-2]
            if self.only_change == False or self.previous_generation != status.generation:
                self.__write_rf_status(id, status)
                if statuses[-1].state != 'Active':
                    self.__write_rf_status(id, statuses[-1])
                self.previous_generation = status.generation

    def __write_rf_status(self, id, status):
        msg = ("%s,%s,%d,%d,%f,%f,%d,%s,%s,%s") % (
            datetime.now(),
            status.state,
            status.generation,
            status.total_generations,
            status.validation_set_error,
            status.training_set_error,
            status.generation_seconds,
            status.start_date_time,
            status.estimated_date_time,
            id
        )
        self.write_string(msg)

    def write_xgboost_statues(self, id, statuses):
        self.write_random_forest_statues(id, statuses)

    def write_string(self, msg, clear=False):
        if isinstance(self.log_file, str):
            mode = 'w' if clear else 'a'
            with open(self.log_file, mode=mode, encoding='utf-8', buffering=1) as f:
                f.write(msg+'\n')
        else:
            self.log_file.write(msg+'\n')
            self.log_file.flush()
